/* Copyright (c) 2000-2006 ActiveState Software Inc.
   See the file LICENSE.txt for licensing information. */

/*
 * This file contains the functions that launch browsers for e.g.
 * the language reference pages.
 *
 * A utility function ko.browse.openUrlInDefaultBrowser(url)
 * is also provided, which can be used from elsewhere in Komodo.
 */

if (typeof(ko)=='undefined') {
    var ko = {};
}

ko.browse = {};
(function() {
var bundle = Components.classes["@mozilla.org/intl/stringbundle;1"]
        .getService(Components.interfaces.nsIStringBundleService)
        .createBundle("chrome://komodo/locale/browse.properties");
/**
 * open the given url or complain appropriately
 *
 * @param {String} url
 * @param {String} browser optional, retreived from prefs if not used
 */
this.openUrlInDefaultBrowser = function browse_OpenUrlInDefaultBrowser(url, browser) {
    var koWebbrowser = Components.classes['@activestate.com/koWebbrowser;1'].
                       getService(Components.interfaces.koIWebbrowser);
    var prefs = Components.classes['@activestate.com/koPrefService;1'].
            getService(Components.interfaces.koIPrefService).prefs;

    if (typeof(browser) == 'undefined' || !browser) {
        browser = prefs.getStringPref('browser');
    }
    var infoService = Components.classes['@activestate.com/koInfoService;1'].
                      getService(Components.interfaces.koIInfoService);
    var platform = infoService.platform;
    if (!platform.match(/^win/) && !platform.match(/^darwin/)
        && browser == '')
    {
        // Don't guess, since launching e.g. Mozilla can have side effects
        var browsers = koWebbrowser.get_possible_browsers(new Object());
        var answer = ko.dialogs.selectFromList(
                bundle.GetStringFromName("chooseDefaultBrowser.message"),
                bundle.GetStringFromName("selectTheBrowserToOpen.message"),
                browsers,
                "one"); // selectionCondition
        if (answer == null) {
            return;
        }
        prefs.setStringPref("browser", answer[0]);
        browser = answer[0];
    }
    var ret = false;
    try {
        if (browser) {
            ret = koWebbrowser.open_new_browser(url,browser);
        } else {
            ret = koWebbrowser.open_new(url);
        }
    } catch (e) {
        log.exception(e);
    }
    if (!ret) {
        if (browser) {
            ko.dialogs.alert(bundle.formatStringFromName(
                "couldNotOpenTheBrowserAt.alert", [browser], 1));
        } else {
            ko.dialogs.alert(bundle.formatStringFromName(
                "couldNotFindAGraphicalBrowser.alert", [url], 1));
        }
    }
}

/**
 * show a list of keybindings in the browser
 */
this.showKeybindings = function browse_ShowKeybindings()
{
    try {
        var text = ko.keybindings.manager.makeCurrentKeyBindingTable();
        var tmpFileSvc = Components.classes["@activestate.com/koFileService;1"]
                         .getService(Components.interfaces.koIFileService)
        var outputTmpFile = tmpFileSvc.makeTempFile(".html","w");
        //dump('outputTmpFile name is '+outputTmpFile.URI+'\n');
        outputTmpFile.puts(text);
        outputTmpFile.flush();
        outputTmpFile.close();
        browse_OpenUrlInDefaultBrowser(outputTmpFile.URI);
    } catch(e) {
        log.exception(e);
    }
}

/**
 * show a list of command id's in the browser
 */
this.showCommandIds = function browse_ShowCommandIds()
{
    try {
        var text = ko.keybindings.manager.makeCommandIdTable();
        var tmpFileSvc = Components.classes["@activestate.com/koFileService;1"]
                         .getService(Components.interfaces.koIFileService)
        var outputTmpFile = tmpFileSvc.makeTempFile(".html","w");
        //dump('outputTmpFile name is '+outputTmpFile.URI+'\n');
        outputTmpFile.puts(text);
        outputTmpFile.flush();
        outputTmpFile.close();
        browse_OpenUrlInDefaultBrowser(outputTmpFile.URI);
    } catch(e) {
        log.exception(e);
    }
}

/**
 * show the url defined in "localHelpFile" in an instance of koIAppInfoEx
 *
 * @param {String} app the app identifier from the CID of a koIAppInfoEx
 *                 implementation (eg. @activestate.com/koAppInfoEx?app=Perl)
 */
this.localHelp = function(app) {
    var info = Components.classes["@activestate.com/koAppInfoEx?app="+app+";1"]
                      .getService(Components.interfaces.koIAppInfoEx);
    browse_OpenUrlInDefaultBrowser(info.localHelpFile);
}

/**
 * show the url defined in "webHelpURL" in an instance of koIAppInfoEx
 *
 * @param {String} app the app identifier from the CID of a koIAppInfoEx
 *                 implementation (eg. @activestate.com/koAppInfoEx?app=Perl)
 */
this.webHelp = function(app) {
    var info = Components.classes["@activestate.com/koAppInfoEx?app="+app+";1"]
                      .getService(Components.interfaces.koIAppInfoEx);
    browse_OpenUrlInDefaultBrowser(info.webHelpURL);
}

/**
 * show mailing list archives on ASPN that are related to the topic
 *
 * @param {String} topic
 */
this.aspnMailingList = function(topic) {
    var url = "http://code.activestate.com/lists/#"+topic;
    browse_OpenUrlInDefaultBrowser(url);
}

/**
 * Hide or show the local help entries in the Help->Languages popup
 * depending on whether an actual help file to launch can be found.
 */
this.updateHelpLanguagesPopup = function browse_UpdateHelpLanguagesPopup() {
    var perlInfoSvc = Components.classes['@activestate.com/koAppInfoEx?app=Perl;1'].
                      getService(Components.interfaces.koIAppInfoEx);
    var perlWidget = document.getElementById("menu_helpPerlRef_Local");
    var perlHelpFile = perlInfoSvc.localHelpFile;
    if (perlHelpFile) {
        perlWidget.removeAttribute("hidden");
        perlWidget.removeAttribute("collapsed");
    } else {
        perlWidget.setAttribute("hidden", true);
        perlWidget.setAttribute("collapsed", true);
    }

    var pythonInfoSvc = Components.classes['@activestate.com/koAppInfoEx?app=Python;1'].
                        getService(Components.interfaces.koIAppInfoEx);
    var pythonWidget = document.getElementById("menu_helpPythonRef_Local");
    var pythonHelpFile = pythonInfoSvc.localHelpFile;
    if (pythonHelpFile) {
        pythonWidget.removeAttribute("hidden");
        pythonWidget.removeAttribute("collapsed");
    } else {
        pythonWidget.setAttribute("hidden", true);
        pythonWidget.setAttribute("collapsed", true);
    }

    var tclInfoSvc = Components.classes['@activestate.com/koAppInfoEx?app=Tcl;1'].
                     getService(Components.interfaces.koIAppInfoEx);
    var tclWidget = document.getElementById("menu_helpTclRef_Local");
    var tclHelpFile = tclInfoSvc.localHelpFile;
    if (tclHelpFile) {
        tclWidget.removeAttribute("hidden");
        tclWidget.removeAttribute("collapsed");
    } else {
        tclWidget.setAttribute("hidden", true);
        tclWidget.setAttribute("collapsed", true);
    }
}


// XXX move these to a properties file or prefs.js
var tag2uri = {
    'mailLists': "http://code.activestate.com/lists/komodo-discuss/",
    'home': "http://www.ActiveState.com/",
    'aspn': "http://code.activestate.com/",
    'community': "http://community.activestate.com/products/Komodo",
    'bugs': "http://bugs.activestate.com/query.cgi?format=specific&product=Komodo",
    'contactus': "http://www.activestate.com/company/contact-us"
};

/**
 * browse to a predefined url on activestate.com  see tag2uri in ko.browse
 */
this.browseTag = function(tag) {
    if (tag in tag2uri) {
        ko.browse.openUrlInDefaultBrowser(tag2uri[tag]);
    } else {
        ko.statusBar.AddMessage(
            "ko.browse.browseTag error: unknown tag '"+tag+"'",
            "browse", 3000, true);
    }
}
}).apply(ko.browse);

/**
 * @deprecated since 7.0
 */
ko.logging.globalDeprecatedByAlternative("browse_OpenUrlInDefaultBrowser", "ko.browse.openUrlInDefaultBrowser");
ko.logging.globalDeprecatedByAlternative("browse_ShowCommandIds", "ko.browse.showCommandIds");
ko.logging.globalDeprecatedByAlternative("browse_ShowKeybindings", "ko.browse.showKeybindings");
ko.logging.globalDeprecatedByAlternative("browse_UpdateHelpLanguagesPopup", "ko.browse.updateHelpLanguagesPopup");

ko.logging.globalDeprecatedByAlternative("browse_PerlRef_Local", "ko.browse.localHelp('Perl')");
ko.logging.globalDeprecatedByAlternative("browse_PerlRef_Web", "ko.browse.webHelp('Perl')");
ko.logging.globalDeprecatedByAlternative("browse_PerlMailingLists", "ko.browse.aspnMailingList('Perl')");

ko.logging.globalDeprecatedByAlternative("browse_PythonRef_Local", "ko.browse.localHelp('Python')");
ko.logging.globalDeprecatedByAlternative("browse_PythonRef_Web", "ko.browse.webHelp('Python')");
ko.logging.globalDeprecatedByAlternative("browse_PythonMailingLists", "ko.browse.aspnMailingList('Python')");

ko.logging.globalDeprecatedByAlternative("browse_PHPRef_Web", "ko.browse.webHelp('PHP')");
ko.logging.globalDeprecatedByAlternative("browse_PHPMailingLists", "ko.browse.aspnMailingList('PHP')");

ko.logging.globalDeprecatedByAlternative("browse_RubyRef_Web", "ko.browse.webHelp('Ruby')");
ko.logging.globalDeprecatedByAlternative("browse_RubyMailingLists", "ko.browse.aspnMailingList('Ruby')");

ko.logging.globalDeprecatedByAlternative("browse_TclRef_Local", "ko.browse.localHelp('Tcl')");
ko.logging.globalDeprecatedByAlternative("browse_TclRef_Web", "ko.browse.webHelp('Tcl')");
ko.logging.globalDeprecatedByAlternative("browse_TclMailingLists", "ko.browse.aspnMailingList('Tcl')");

ko.logging.globalDeprecatedByAlternative("browse_XSLTMailingLists", "ko.browse.aspnMailingList('XSLT')");

ko.logging.globalDeprecatedByAlternative("browse_MailLists", "ko.browse.browseTag('XSLT')");
ko.logging.globalDeprecatedByAlternative("browse_ActiveState", "ko.browse.browseTag('home')");
ko.logging.globalDeprecatedByAlternative("browse_ASPN", "ko.browse.browseTag('aspn')");
ko.logging.globalDeprecatedByAlternative("browse_Support", "ko.browse.browseTag('community')");
ko.logging.globalDeprecatedByAlternative("browse_Bugs", "ko.browse.browseTag('bugs')");
